# !/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.30 00:00:00                  #
# ================================================== #

from pygpt_net.core.types import (
    MODE_AGENT,
    MODE_AGENT_LLAMA,
    MODE_ASSISTANT,
    MODE_EXPERT,
    MODE_IMAGE,
    MODE_LLAMA_INDEX,
    MODE_VISION,
    MODE_COMPUTER,
    MODE_AGENT_OPENAI,
    MODE_COMPLETION,
)
from pygpt_net.core.tabs.tab import Tab
from pygpt_net.core.events import Event
from pygpt_net.utils import trans


class Mode:
    def __init__(self, window=None):
        """
        UI mode switch controller

        :param window: Window instance
        """
        self.window = window

    def update(self):
        """Update mode, model, preset and rest of the toolbox"""

        mode = self.window.core.config.data['mode']

        # presets
        if mode != MODE_ASSISTANT:
            self.window.ui.nodes['presets.widget'].setVisible(True)
        else:
            self.window.ui.nodes['presets.widget'].setVisible(False)

        if mode != MODE_COMPUTER:
            self.window.ui.nodes['env.widget'].setVisible(False)
        else:
            self.window.ui.nodes['env.widget'].setVisible(True)

        # presets: labels
        if mode == MODE_AGENT:
            self.window.ui.nodes['preset.agents.label'].setVisible(True)
            self.window.ui.nodes['preset.experts.label'].setVisible(False)
            self.window.ui.nodes['preset.presets.label'].setVisible(False)
        elif mode == MODE_AGENT_LLAMA:
            self.window.ui.nodes['preset.agents.label'].setVisible(True)
            self.window.ui.nodes['preset.experts.label'].setVisible(False)
            self.window.ui.nodes['preset.presets.label'].setVisible(False)
        elif mode == MODE_AGENT_OPENAI:
            self.window.ui.nodes['preset.agents.label'].setVisible(True)
            self.window.ui.nodes['preset.experts.label'].setVisible(False)
            self.window.ui.nodes['preset.presets.label'].setVisible(False)
        elif mode == MODE_EXPERT:
            self.window.ui.nodes['preset.agents.label'].setVisible(False)
            self.window.ui.nodes['preset.experts.label'].setVisible(True)
            self.window.ui.nodes['preset.presets.label'].setVisible(False)
        else:
            self.window.ui.nodes['preset.agents.label'].setVisible(False)
            self.window.ui.nodes['preset.experts.label'].setVisible(False)
            self.window.ui.nodes['preset.presets.label'].setVisible(True)

        # presets: experts
        if mode == MODE_EXPERT:
            self.window.ui.nodes['preset.editor.description'].setVisible(True)
            self.window.ui.nodes['preset.editor.remote_tools'].setVisible(True)
        else:
            self.window.ui.nodes['preset.editor.description'].setVisible(False)
            self.window.ui.nodes['preset.editor.remote_tools'].setVisible(False)

        if mode == MODE_COMPLETION:
            self.window.ui.nodes['preset.editor.ai_name'].setVisible(True)
            self.window.ui.nodes['preset.editor.user_name'].setVisible(True)
        else:
            self.window.ui.nodes['preset.editor.ai_name'].setVisible(False)
            self.window.ui.nodes['preset.editor.user_name'].setVisible(False)

        if mode == MODE_AGENT_OPENAI:
            self.window.ui.nodes['preset.editor.agent_provider_openai'].setVisible(True)
        else:
            self.window.ui.nodes['preset.editor.agent_provider_openai'].setVisible(False)

        # presets: editor
        if mode == MODE_AGENT:
            self.window.ui.nodes['preset.editor.temperature'].setVisible(True)
            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(False)
            self.window.ui.nodes['preset.editor.functions'].setVisible(False)
            self.window.ui.nodes['preset.editor.modes'].setVisible(False)
            self.window.ui.nodes['preset.editor.experts'].setVisible(True)
            self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt.agent"))
            self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(False)
        elif mode == MODE_AGENT_LLAMA:
            self.window.ui.nodes['preset.editor.temperature'].setVisible(False)
            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(True)
            self.window.ui.nodes['preset.editor.functions'].setVisible(False)
            self.window.ui.nodes['preset.editor.modes'].setVisible(False)
            self.window.ui.nodes['preset.editor.experts'].setVisible(False)
            self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt.agent_llama"))
            self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(True)
        elif mode == MODE_AGENT_OPENAI:
            self.window.ui.nodes['preset.editor.temperature'].setVisible(False)
            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(False)
            self.window.ui.nodes['preset.editor.functions'].setVisible(False)
            self.window.ui.nodes['preset.editor.modes'].setVisible(False)
            self.window.ui.nodes['preset.editor.experts'].setVisible(True)
            self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt.agent_llama"))
            self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(True)
        else:
            self.window.ui.nodes['preset.editor.temperature'].setVisible(True)
            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(False)
            self.window.ui.nodes['preset.editor.functions'].setVisible(True)
            self.window.ui.nodes['preset.editor.modes'].setVisible(True)
            self.window.ui.nodes['preset.editor.experts'].setVisible(False)
            self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt"))
            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(False)

            if mode == MODE_ASSISTANT:
                self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(True)
                self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
            else:
                self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
                self.window.ui.nodes['preset.tool.function.label.all'].setVisible(True)

        # presets: clear
        """
        self.window.ui.nodes['preset.clear'].setVisible(False)
        if mode in [MODE_IMAGE, MODE_ASSISTANT]:
            self.window.ui.nodes['preset.clear'].setVisible(False)
        else:
            self.window.ui.nodes['preset.clear'].setVisible(True)
        """

        # presets: use
        """
        if mode == MODE_IMAGE:
            self.window.ui.nodes['preset.use'].setVisible(True)
        else:
            self.window.ui.nodes['preset.use'].setVisible(False)
        """

        # img options
        if mode == MODE_IMAGE:
            self.window.ui.nodes['dalle.options'].setVisible(True)
        else:
            self.window.ui.nodes['dalle.options'].setVisible(False)

        # agent options
        if mode in [MODE_AGENT]:
            self.window.ui.nodes['agent.options'].setVisible(True)
        else:
            self.window.ui.nodes['agent.options'].setVisible(False)

        # agent llama options
        if mode in [MODE_AGENT_LLAMA, MODE_AGENT_OPENAI]:
            self.window.ui.nodes['agent_llama.options'].setVisible(True)
        else:
            self.window.ui.nodes['agent_llama.options'].setVisible(False)

        """
        # agent llama sys prompt
        if mode in [MODE_AGENT_LLAMA]:
            self.window.ui.nodes['preset.prompt'].setVisible(False)
        else:
            self.window.ui.nodes['preset.prompt'].setVisible(True)
        """

        # assistants list
        if mode == MODE_ASSISTANT:
            self.window.ui.nodes['assistants.widget'].setVisible(True)
        else:
            self.window.ui.nodes['assistants.widget'].setVisible(False)

        # indexes list
        if mode == MODE_LLAMA_INDEX:
            # self.window.ui.nodes['indexes.widget'].setVisible(True)
            self.window.ui.nodes['idx.options'].setVisible(True)
        else:
            # self.window.ui.nodes['indexes.widget'].setVisible(False)
            self.window.ui.nodes['idx.options'].setVisible(False)

        # stream mode
        if mode in [MODE_IMAGE]:

            self.window.ui.nodes['input.stream'].setVisible(False)
        else:
            self.window.ui.nodes['input.stream'].setVisible(True)

        # vision
        show = self.is_vision(mode)
        self.window.ui.menu['menu.video'].menuAction().setVisible(show)
        self.window.ui.nodes['icon.video.capture'].setVisible(show)
        # self.window.ui.nodes['vision.capture.options'].setVisible(show)
        self.window.ui.nodes['attachments.capture_clear'].setVisible(show)

        # attachments
        show = self.are_attachments(mode)
        self.window.ui.tabs['input'].setTabVisible(1, show)  # attachments

        # uploaded files
        if mode == MODE_ASSISTANT:
            self.window.ui.tabs['input'].setTabVisible(2, True)
            self.window.ui.tabs['input'].setTabVisible(3, False)
        else:
            if mode != MODE_IMAGE:
                self.window.ui.tabs['input'].setTabVisible(2, False)
                self.window.ui.tabs['input'].setTabVisible(3, True)
            else:
                self.window.ui.tabs['input'].setTabVisible(2, False)
                self.window.ui.tabs['input'].setTabVisible(3, False)

        # chat footer
        self.toggle_chat_footer()

    def toggle_chat_footer(self):
        """Toggle chat footer"""
        if self.window.controller.ui.tabs.get_current_type() != Tab.TAB_CHAT:
            self.hide_chat_footer()
        else:
            self.show_chat_footer()

    def is_vision(self, mode: str) -> bool:
        """
        Check if vision is allowed

        :param mode: current mode
        :return: True if vision is allowed
        """
        if self.window.controller.ui.vision.has_vision():
            return True

        if self.window.controller.painter.is_active():
            return True

        # event: UI: vision
        value = False
        event = Event(Event.UI_VISION, {
            'mode': mode,
            'value': value,
        })
        self.window.dispatch(event)
        return event.data['value']

    def are_attachments(self, mode: str) -> bool:
        """
        Check if attachments are allowed

        :param mode: current mode
        :return: True if attachments are allowed
        """
        # event: UI: attachments
        # value = False
        event = Event(Event.UI_ATTACHMENTS, {
            'mode': mode,
            'value': True,
        })
        self.window.dispatch(event)
        return event.data['value']

    def show_chat_footer(self):
        """Show chat footer"""
        self.window.ui.nodes['chat.footer'].setVisible(True)

    def hide_chat_footer(self):
        """Hide chat footer"""
        self.window.ui.nodes['chat.footer'].setVisible(False)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.30 00:00:00                  #
# ================================================== #

from typing import Dict, Any, Tuple

from pygpt_net.item.ctx import CtxItem


class BaseAgent:
    def __init__(self, *args, **kwargs):
        self.id = ""
        self.type = ""
        self.mode = ""
        self.name = ""

    def get_mode(self) -> str:
        """
        Return Agent mode

        :return: Agent mode
        """
        return self.mode

    def get_agent(
            self,
            window,
            kwargs: Dict[str, Any]
    ) -> Any:
        """
        Return Agent provider instance

        :param window: window instance
        :param kwargs: keyword arguments
        :return: Agent provider instance
        """
        pass

    async def run(
            self,
            window,
            agent_kwargs: Dict[str, Any] = None,
            previous_response_id: str = None,
            messages: list = None,
            ctx: CtxItem = None,
            stream: bool = False,
            stopped: callable = None,
            on_step: callable = None,
            on_stop: callable = None,
            on_error: callable = None,
    ) -> Tuple[str, str]:
        """
        Run agent (async)

        :param window: Window instance
        :param agent_kwargs: Additional agent parameters
        :param previous_response_id: ID of the previous response (if any)
        :param messages: Conversation messages
        :param ctx: Context item
        :param stream: Whether to stream output
        :param stopped: Callback for stop event received from the user
        :param on_step: Callback for each step
        :param on_stop: Callback for stopping the process
        :param on_error: Callback for error handling
        :return: Final output and response ID
        """
        pass


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.30 00:00:00                  #
# ================================================== #

import base64
from typing import Tuple

from agents import HandoffOutputItem

from pygpt_net.item.ctx import CtxItem

from openai.types.responses import (
    ResponseTextDeltaEvent,
    ResponseCreatedEvent,
    ResponseCodeInterpreterCallCodeDeltaEvent,
    ResponseOutputItemAddedEvent,
    ResponseCompletedEvent,
    ResponseOutputItemDoneEvent,
)

class StreamHandler:
    def __init__(self, window, on_step: callable):
        self.window = window
        self.on_step = on_step
        self.buffer = ""
        self.begin = True
        self.response_id = None
        self.files = []
        self.finished = False
        self.files_handled = False
        self.code_block = False

    def handle(self, event, ctx: CtxItem) -> Tuple[str, str]:
        """
        Unpack agent response and set context

        :param event: Event - event containing response data
        :param ctx: CtxItem - context item to set the response data
        :return: Final output string, response ID
        """
        img_path = self.window.core.image.gen_unique_path(ctx)
        is_image = False
        if event.type == "raw_response_event" and isinstance(event.data, ResponseCreatedEvent):
            self.response_id = event.data.response.id
        elif event.type == "raw_response_event" and isinstance(event.data, ResponseTextDeltaEvent):
            ctx.stream = event.data.delta
            if self.code_block:
                ctx.stream = "\n```\n" + ctx.stream
                self.code_block = False
            self.on_step(ctx, self.begin)
            self.buffer += ctx.stream
            self.begin = False
        elif event.type == "raw_response_event" and isinstance(event.data, ResponseOutputItemAddedEvent):
            if event.data.item.type == "code_interpreter_call":
                self.code_block = True
                ctx.stream = "\n\n**Code interpreter**\n```python\n"
                self.on_step(ctx, self.begin)
                self.buffer += ctx.stream
                self.begin = False
        elif event.type == "raw_response_event" and isinstance(event.data, ResponseOutputItemDoneEvent):
            if event.data.item.type == "image_generation_call":
                image_base64 = event.data.item.result
                image_bytes = base64.b64decode(image_base64)
                with open(img_path, "wb") as f:
                    f.write(image_bytes)
                is_image = True
        elif event.type == "raw_response_event" and isinstance(event.data, ResponseCodeInterpreterCallCodeDeltaEvent):
            ctx.stream = event.data.delta
            self.on_step(ctx, self.begin)
            self.buffer += ctx.stream
            self.begin = False
        elif event.type == "raw_response_event" and isinstance(event.data, ResponseCompletedEvent):
            for item in event.data.response.output:
                if item.type == "message":
                    for content in item.content:
                        if content.annotations:
                            for annotation in content.annotations:
                                # url citation
                                if annotation.type == "url_citation":
                                    if ctx.urls is None:
                                        ctx.urls = []
                                    ctx.urls.append(annotation.url)
                                # container file citation
                                elif annotation.type == "container_file_citation":
                                    container_id = annotation.container_id
                                    file_id = annotation.file_id
                                    self.files.append({
                                        "container_id": container_id,
                                        "file_id": file_id,
                                    })
                elif item.type == "code_interpreter_call":
                    if self.code_block:
                        ctx.stream = "\n```\n"
                        self.code_block = False
                    self.on_step(ctx, self.begin)
                    self.buffer += ctx.stream
                    self.begin = False


            self.finished = True

        elif event.type == "run_item_stream_event":
            if isinstance(event.item, HandoffOutputItem):
                ctx.stream = f"\n\n**Handoff to: {event.item.target_agent.name}**\n\n"
                self.on_step(ctx, self.begin)
                self.buffer += ctx.stream
                self.begin = False

        # append images
        if is_image:
            self.window.core.debug.info("[chat] Image generation call found")
            ctx.images = [img_path]  # save image path to ctx

        # if files from container are found, download them and append to ctx
        if self.finished and not self.files_handled and self.files:
            self.files_handled = True
            self.window.core.debug.info("[chat] Container files found, downloading...")
            try:
                self.window.core.gpt.container.download_files(ctx, self.files)
            except Exception as e:
                self.window.core.debug.error(f"[chat] Error downloading container files: {e}")

        return self.buffer, self.response_id


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.30 00:00:00                  #
# ================================================== #

from typing import Dict, Any, Tuple

from pygpt_net.core.bridge import BridgeContext
from pygpt_net.core.types import (
    AGENT_MODE_OPENAI,
    AGENT_TYPE_OPENAI,
)

from agents import (
    Agent as OpenAIAgent,
    Runner,
    RunConfig,
    ModelSettings,
)

from pygpt_net.item.ctx import CtxItem
from pygpt_net.item.model import ModelItem
from pygpt_net.item.preset import PresetItem

from .client import get_custom_model_provider, set_openai_env
from .remote_tools import get_remote_tools, is_computer_tool
from .response import StreamHandler
from ..base import BaseAgent

class Agent(BaseAgent):
    def __init__(self, *args, **kwargs):
        super(Agent, self).__init__(*args, **kwargs)
        self.id = "openai_agent_experts"
        self.type = AGENT_TYPE_OPENAI
        self.mode = AGENT_MODE_OPENAI
        self.name = "OpenAI Agents: with experts"

    def get_agent(self, window, kwargs: Dict[str, Any]):
        """
        Return Agent provider instance

        :param window: window instance
        :param kwargs: keyword arguments
        :return: Agent provider instance
        """
        context = kwargs.get("context", BridgeContext())
        preset = context.preset
        system_prompt = kwargs.get("system_prompt", "")
        agent_name = preset.name if preset else "Agent"
        model = kwargs.get("model", ModelItem())
        tools = kwargs.get("function_tools", [])
        handoffs = kwargs.get("handoffs", [])

        # append remote tools if available
        tool_kwargs = {
            "window": window,
            "model": model,
            "preset": preset,
            "is_expert_call": False,
        }
        remote_tools = get_remote_tools(**tool_kwargs)
        all_tools = remote_tools + tools

        kwargs = {
            "name": agent_name,
            "instructions": system_prompt,
            "model": model.id,
            "tools": all_tools,
            "handoffs": handoffs,
        }
        if is_computer_tool(**tool_kwargs):
            kwargs["model_settings"] = ModelSettings(truncation="auto")
        return OpenAIAgent(**kwargs)


    def get_expert(
            self,
            window,
            prompt: str,
            model: ModelItem,
            preset: PresetItem = None,
            tools: list = None,
    ) -> OpenAIAgent:
        """
        Return Agent provider instance

        :param window: window instance
        :param prompt: Expert prompt
        :param model: Model item
        :param preset: Preset item
        :param tools: List of function tools
        :return: Agent provider instance
        """
        agent_name = preset.name if preset else "Agent"

        # append remote tools if available
        tool_kwargs = {
            "window": window,
            "model": model,
            "preset": preset,
            "is_expert_call": True,
        }
        remote_tools = get_remote_tools(**tool_kwargs)
        all_tools = remote_tools + tools
        kwargs = {
            "name": agent_name,
            "instructions": prompt,
            "model": model.id,
            "tools": all_tools,
        }
        if is_computer_tool(**tool_kwargs):
            kwargs["model_settings"] = ModelSettings(truncation="auto")
        return OpenAIAgent(**kwargs)

    async def run(
            self,
            window: Any = None,
            agent_kwargs: Dict[str, Any] = None,
            previous_response_id: str = None,
            messages: list = None,
            ctx: CtxItem = None,
            stream: bool = False,
            stopped: callable = None,
            on_step: callable = None,
            on_stop: callable = None,
            on_error: callable = None,
    ) -> Tuple[str, str]:
        """
        Run agent (async)

        :param window: Window instance
        :param agent_kwargs: Additional agent parameters
        :param previous_response_id: ID of the previous response (if any)
        :param messages: Conversation messages
        :param ctx: Context item
        :param stream: Whether to stream output
        :param stopped: Callback for stop event received from the user
        :param on_step: Callback for each step
        :param on_stop: Callback for stopping the process
        :param on_error: Callback for error handling
        :return: Final output and response ID
        """
        final_output = ""
        response_id = None
        model = agent_kwargs.get("model", ModelItem())
        verbose = agent_kwargs.get("verbose", False)
        context = agent_kwargs.get("context", BridgeContext())
        tools = agent_kwargs.get("function_tools", [])
        preset = context.preset if context else None

        # prepare experts as agents
        experts = []
        uuids = preset.experts
        for uuid in uuids:
            expert = window.core.presets.get_by_uuid(uuid)
            if expert:
                experts.append(expert)

        expert_agents = []
        for expert in experts:
            model = window.core.models.get(expert.model)
            expert_agent = self.get_expert(
                window=window,
                prompt=expert.prompt,
                model=model,
                preset=expert,
                tools=tools,
            )
            expert_agents.append(expert_agent)
            if verbose:
                print(f"Adding expert: {expert.name} ({model.id})")

        agent_kwargs["handoffs"] = expert_agents
        agent = self.get_agent(window, agent_kwargs)

        kwargs = {
            "input": messages,
        }
        if model.provider != "openai":
            custom_provider = get_custom_model_provider(window, model)
            kwargs["run_config"] = RunConfig(model_provider=custom_provider)
        else:
            set_openai_env(window)
            if previous_response_id:
                kwargs["previous_response_id"] = previous_response_id

        if not stream:
            result = await Runner.run(
                agent,
                **kwargs
            )
            final_output, last_response_id = window.core.gpt.responses.unpack_agent_response(result, ctx)
            response_id = result.last_response_id
            if verbose:
                print("Final response:", result)
        else:
            result = Runner.run_streamed(
                agent,
                **kwargs
            )
            handler = StreamHandler(window, on_step)
            async for event in result.stream_events():
                if stopped():
                    on_stop(ctx)
                    break
                final_output, response_id = handler.handle(event, ctx)

        return final_output, response_id





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.28 16:00:00                  #
# ================================================== #

# from langchain_community.llms import HuggingFaceHub

from pygpt_net.core.types import (
    MODE_LANGCHAIN,
)
from pygpt_net.provider.llms.base import BaseLLM
from pygpt_net.item.model import ModelItem


class HuggingFaceLLM(BaseLLM):
    def __init__(self, *args, **kwargs):
        super(HuggingFaceLLM, self).__init__(*args, **kwargs)
        self.id = "huggingface"
        self.name = "HuggingFace"
        self.type = []

    def completion(
            self,
            window,
            model: ModelItem,
            stream: bool = False
    ):
        """
        Return LLM provider instance for completion

        :param window: window instance
        :param model: model instance
        :param stream: stream mode
        :return: LLM provider instance

        args = self.parse_args(model.langchain)
        if "model" not in args:
            args["model"] = model.id
        return HuggingFaceHub(**args)
        """
        pass

    def chat(
            self,
            window,
            model: ModelItem,
            stream: bool = False
    ):
        """
        Return LLM provider instance for chat

        :param window: window instance
        :param model: model instance
        :param stream: stream mode
        :return: LLM provider instance
        """
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.30 18:00:00                  #
# ================================================== #

import base64
import datetime
import os
from typing import Optional, Dict, Any

import requests

from PySide6.QtCore import QObject, Signal, QRunnable, Slot

from pygpt_net.core.events import KernelEvent
from pygpt_net.core.bridge.context import BridgeContext
from pygpt_net.item.ctx import CtxItem
from pygpt_net.utils import trans


class Image:

    MODE_GENERATE = "generate"
    MODE_EDIT = "edit"

    def __init__(self, window=None):
        """
        Image generation core

        :param window: Window instance
        """
        self.window = window
        self.worker = None

    def generate(
            self,
            context: BridgeContext,
            extra: Optional[Dict[str, Any]] = None,
            sync: bool = True
    ):
        """
        Call images API

        :param context: Bridge context
        :param extra: Extra arguments
        :param sync: Synchronous mode
        """
        prompt = context.prompt
        ctx = context.ctx
        model = context.model
        num = extra.get("num", 1)
        inline = extra.get("inline", False)
        sub_mode = self.MODE_GENERATE

        # if attachments then switch mode to EDIT
        attachments = context.attachments
        if attachments and len(attachments) > 0:
            sub_mode = self.MODE_EDIT

        if ctx is None:
            ctx = CtxItem()  # create empty context

        prompt_model = self.window.core.models.from_defaults()
        tmp_model = self.window.core.config.get('img_prompt_model')
        if self.window.core.models.has(tmp_model):
            prompt_model = self.window.core.models.get(tmp_model)

        # worker
        self.worker = ImageWorker()
        self.worker.window = self.window
        self.worker.client = self.window.core.gpt.get_client()
        self.worker.ctx = ctx
        self.worker.mode = sub_mode  # mode can be "generate" or "edit"
        self.worker.attachments = attachments  # attachments for edit mode
        self.worker.raw = self.window.core.config.get('img_raw')
        self.worker.model = model.id  # model ID for generate image, e.g. "dall-e-3"
        self.worker.model_prompt = prompt_model  # model for generate prompt, not image!
        self.worker.input_prompt = prompt
        self.worker.system_prompt = self.window.core.prompt.get('img')
        self.worker.num = num
        self.worker.inline = inline

        # config
        if self.window.core.config.has('img_quality'):
            self.worker.quality = self.window.core.config.get('img_quality')
        if self.window.core.config.has('img_resolution'):
            self.worker.resolution = self.window.core.config.get('img_resolution')

        # signals
        self.worker.signals.finished.connect(self.window.core.image.handle_finished)
        self.worker.signals.finished_inline.connect(self.window.core.image.handle_finished_inline)
        self.worker.signals.status.connect(self.window.core.image.handle_status)
        self.worker.signals.error.connect(self.window.core.image.handle_error)

        # sync
        if sync:
            self.worker.run()
            return True

        # check if async allowed
        if not self.window.controller.kernel.async_allowed(ctx):
            self.worker.run()
            return True

        # start
        self.window.dispatch(KernelEvent(KernelEvent.STATE_BUSY, {
            "id": "img",
        }))
        self.window.threadpool.start(self.worker)
        return True


class ImageSignals(QObject):
    finished = Signal(object, list, str)  # ctx, paths, prompt
    finished_inline = Signal(object, list, str)  # ctx, paths, prompt
    status = Signal(object)  # status message
    error = Signal(object)  # error message


class ImageWorker(QObject, QRunnable):
    def __init__(self, *args, **kwargs):
        QObject.__init__(self)
        QRunnable.__init__(self)
        self.signals = ImageSignals()
        self.args = args
        self.kwargs = kwargs
        self.window = None
        self.client = None
        self.ctx = None
        self.raw = False
        self.mode = Image.MODE_GENERATE  # default mode is generate
        self.model = "dall-e-3"
        self.quality = "standard"
        self.resolution = "1792x1024"
        self.attachments = {}  # attachments for edit mode
        self.model_prompt = None
        self.input_prompt = None
        self.system_prompt = None
        self.inline = False
        self.num = 1
        self.allowed_max_num = {
            "dall-e-2": 4,
            "dall-e-3": 1,
            "gpt-image-1": 1,
        }
        self.allowed_resolutions = {
            "dall-e-2": [
                "1024x1024",
                "512x512",
                "256x256",
            ],
            "dall-e-3": [
                "1792x1024",
                "1024x1792",
                "1024x1024",
            ],
            "gpt-image-1": [
                "1536x1024",
                "1024x1536",
                "1024x1024",
                "auto",
            ],
        }
        self.allowed_quality = {
            "dall-e-2": [
                "standard",
            ],
            "dall-e-3": [
                "standard",
                "hd",
            ],
            "gpt-image-1": [
                "auto",
                "high",
                "medium",
                "low",
            ],
        }

    @Slot()
    def run(self):
        """Run worker"""
        if not self.raw and not self.inline:  # disable on inline and raw modes
            try:
                # call GPT for generate better image generate prompt
                self.signals.status.emit(trans('img.status.prompt.wait'))
                bridge_context = BridgeContext(
                    prompt=self.input_prompt,
                    system_prompt=self.system_prompt,
                    model=self.model_prompt,  # model instance
                    max_tokens=200,
                    temperature=1.0,
                )
                event = KernelEvent(KernelEvent.CALL, {
                    'context': bridge_context,
                    'extra': {},
                })
                self.window.dispatch(event)
                response = event.data.get('response')
                if response is not None and response != "":
                    self.input_prompt = response

            except Exception as e:
                self.signals.error.emit(e)
                self.signals.status.emit(trans('img.status.prompt.error') + ": " + str(e))

        self.signals.status.emit(trans('img.status.generating') + ": {}...".format(self.input_prompt))

        paths = []  # downloaded images paths
        try:
            # check if number of images is supported
            if self.model in self.allowed_max_num:
                if self.num > self.allowed_max_num[self.model]:
                    self.num = self.allowed_max_num[self.model]

            # check if resolution is supported
            resolution = self.resolution
            if self.model in self.allowed_resolutions:
                if resolution not in self.allowed_resolutions[self.model]:
                    resolution = self.allowed_resolutions[self.model][0]

            quality = self.quality
            if self.model in self.allowed_quality:
                if quality not in self.allowed_quality[self.model]:
                    quality = self.allowed_quality[self.model][0]

            # send to API
            response = None
            if self.mode == Image.MODE_GENERATE:
                if self.model == "dall-e-2":
                    response = self.client.images.generate(
                        model=self.model,
                        prompt=self.input_prompt,
                        n=self.num,
                        size=resolution,
                    )
                elif self.model == "dall-e-3" or self.model == "gpt-image-1":
                    response = self.client.images.generate(
                        model=self.model,
                        prompt=self.input_prompt,
                        n=self.num,
                        quality=quality,
                        size=resolution,
                    )
            elif self.mode == Image.MODE_EDIT:
                images = []
                for uuid in self.attachments:
                    attachment = self.attachments[uuid]
                    if attachment.path and os.path.exists(attachment.path):
                        images.append(open(attachment.path, "rb"))
                response = self.client.images.edit(
                    model=self.model,
                    image=images,
                    prompt=self.input_prompt,
                )

            # check response
            if response is None:
                self.signals.status.emit("API Error: empty response")
                return

            # download images
            for i in range(self.num):
                if i >= len(response.data):
                    break

                # generate filename
                name = datetime.date.today().strftime(
                    "%Y-%m-%d") + "_" + datetime.datetime.now().strftime("%H-%M-%S") + "-" \
                       + self.window.core.image.make_safe_filename(self.input_prompt) + "-" + str(i + 1) + ".png"
                path = os.path.join(self.window.core.config.get_user_dir("img"), name)

                msg = trans('img.status.downloading') + " (" + str(i + 1) + " / " + str(self.num) + ") -> " + str(path)
                self.signals.status.emit(msg)

                if response.data[i] is None:
                    self.signals.error.emit("API Error: empty image data")
                    return
                if response.data[i].url:  # dall-e 2 and 3 returns URL
                    res = requests.get(response.data[i].url)
                    data = res.content
                else:  # gpt-image-1 returns base64 encoded image
                    data = base64.b64decode(response.data[i].b64_json)

                # save image
                if data and self.window.core.image.save_image(path, data):
                    paths.append(path)
                else:
                    self.signals.error.emit("Error saving image")

            # send finished signal
            if self.inline:
                self.signals.finished_inline.emit(  # separated signal for inline mode
                    self.ctx,
                    paths,
                    self.input_prompt,
                )
            else:
                self.signals.finished.emit(
                    self.ctx,
                    paths,
                    self.input_prompt,
                )

        except Exception as e:
            self.signals.error.emit(e)
            print(trans('img.status.error') + ": " + str(e))
            return
